@@ -1,7 +1,9 @@
 import argparse
 import configargparse
+from contextlib import contextmanager
 import inspect
 import logging
+import os
 import sys
 import tempfile
 import types
@@ -17,6 +19,9 @@ if sys.version_info >= (3, 0):
 else:
     from StringIO import StringIO
 
+# set COLUMNS to get expected wrapping
+os.environ['COLUMNS'] = '80'
+
 # enable logging to simplify debugging
 logger = logging.getLogger()
 logger.level = logging.DEBUG
@@ -40,6 +45,20 @@ def replace_error_method(arg_parser):
     return arg_parser
 
 
+@contextmanager
+def captured_output():
+    """
+    swap stdout and stderr for StringIO so we can do asserts on outputs.
+    """
+    new_out, new_err = StringIO(), StringIO()
+    old_out, old_err = sys.stdout, sys.stderr
+    try:
+        sys.stdout, sys.stderr = new_out, new_err
+        yield sys.stdout, sys.stderr
+    finally:
+        sys.stdout, sys.stderr = old_out, old_err
+
+
 class TestCase(unittest.TestCase):
 
     def initParser(self, *args, **kwargs):
@@ -58,9 +77,9 @@ class TestCase(unittest.TestCase):
 
         return self.parser
 
-    def assertParseArgsRaises(self, regex, *args, **kwargs):
+    def assertParseArgsRaises(self, regex, args, **kwargs):
         self.assertRaisesRegex(argparse.ArgumentError, regex, self.parse,
-                               *args, **kwargs)
+                               args=args, **kwargs)
 
 
 class TestBasicUseCases(TestCase):
@@ -170,7 +189,7 @@ class TestBasicUseCases(TestCase):
                                    if sys.version_info.major < 3 else
                                    "the following arguments are required: vcf, -g/--my-cfg-file",
                                    args="--genome hg19")
-        self.assertParseArgsRaises("not found: file.txt", args="-g file.txt")
+        self.assertParseArgsRaises("Unable to open config file: file.txt. Error: No such file or director", args="-g file.txt")
 
         # check values after setting args on command line
         config_file2 = tempfile.NamedTemporaryFile(mode="w", delete=True)
@@ -230,8 +249,7 @@ class TestBasicUseCases(TestCase):
         if not use_groups:
             self.assertRegex(self.format_help(),
                 'usage: .* \\[-h\\] --genome GENOME \\[-v\\] -g MY_CFG_FILE\n?'
-                '\\s+\\[-d DBSNP\\]\\s+\\[-f FRMT\\]\\s+vcf \\[vcf ...\\]\n\n' +
-                9*r'(.+\s+)'+  # repeated 8 times because .+ matches atmost 1 line
+                '\\s+\\[-d DBSNP\\]\\s+\\[-f FRMT\\]\\s+vcf \\[vcf ...\\]\n\n'
                 'positional arguments:\n'
                 '  vcf \\s+ Variant file\\(s\\)\n\n'
                 'optional arguments:\n'
@@ -240,12 +258,12 @@ class TestBasicUseCases(TestCase):
                 '  -v\n'
                 '  -g MY_CFG_FILE, --my-cfg-file MY_CFG_FILE\n'
                 '  -d DBSNP, --dbsnp DBSNP\\s+\\[env var: DBSNP_PATH\\]\n'
-                '  -f FRMT, --format FRMT\\s+\\[env var: OUTPUT_FORMAT\\]\n')
+                '  -f FRMT, --format FRMT\\s+\\[env var: OUTPUT_FORMAT\\]\n\n' +
+                7*r'(.+\s*)')
         else:
             self.assertRegex(self.format_help(),
                 'usage: .* \\[-h\\] --genome GENOME \\[-v\\] -g MY_CFG_FILE\n?'
-                '\\s+\\[-d DBSNP\\]\\s+\\[-f FRMT\\]\\s+vcf \\[vcf ...\\]\n\n'+
-                9*r'.+\s+'+  # repeated 8 times because .+ matches atmost 1 line
+                '\\s+\\[-d DBSNP\\]\\s+\\[-f FRMT\\]\\s+vcf \\[vcf ...\\]\n\n'
                 'positional arguments:\n'
                 '  vcf \\s+ Variant file\\(s\\)\n\n'
                 'optional arguments:\n'
@@ -256,7 +274,8 @@ class TestBasicUseCases(TestCase):
                 '  -g MY_CFG_FILE, --my-cfg-file MY_CFG_FILE\n\n'
                 'g2:\n'
                 '  -d DBSNP, --dbsnp DBSNP\\s+\\[env var: DBSNP_PATH\\]\n'
-                '  -f FRMT, --format FRMT\\s+\\[env var: OUTPUT_FORMAT\\]\n')
+                '  -f FRMT, --format FRMT\\s+\\[env var: OUTPUT_FORMAT\\]\n\n' +
+                7*r'(.+\s*)')
 
         self.assertParseArgsRaises("invalid choice: 'ZZZ'",
             args="--genome hg19 -g %s --format ZZZ f.vcf" % config_file2.name)
@@ -270,6 +289,38 @@ class TestBasicUseCases(TestCase):
     def testBasicCase2_WithGroups(self):
         self.testBasicCase2(use_groups=True)
 
+    def testCustomOpenFunction(self):
+        expected_output = 'dummy open called'
+
+        def dummy_open(p):
+            print(expected_output)
+            return open(p)
+
+        self.initParser(config_file_open_func=dummy_open)
+        self.add_arg('--config', is_config_file=True)
+        self.add_arg('--arg1', default=1, type=int)
+
+        with tempfile.NamedTemporaryFile(mode='w', delete=False) as config_file:
+            config_file.write('arg1 2')
+            config_file_path = config_file.name
+
+        with captured_output() as (out, _):
+            args = self.parse('--config {}'.format(config_file_path))
+            self.assertTrue(hasattr(args, 'arg1'))
+            self.assertEqual(args.arg1, 2)
+            output = out.getvalue().strip()
+            self.assertEqual(output, expected_output)
+
+    def testIgnoreHelpArgs(self):
+        p = self.initParser()
+        self.add_arg('--arg1')
+        args, _ = self.parse_known('--arg2 --help', ignore_help_args=True)
+        self.assertEqual(args.arg1, None)
+        self.add_arg('--arg2')
+        args, _ = self.parse_known('--arg2 3 --help', ignore_help_args=True)
+        self.assertEqual(args.arg2, "3")
+        self.assertRaisesRegex(TypeError, "exit", self.parse_known, '--arg2 3 --help', ignore_help_args=False)
+
     def testPositionalAndConfigVarLists(self):
         self.initParser()
         self.add_arg("a")
@@ -280,7 +331,6 @@ class TestBasicUseCases(TestCase):
         self.assertEqual(ns.arg, ['Shell', 'someword', 'anotherword'])
         self.assertEqual(ns.a, "positional_value")
 
-
     def testMutuallyExclusiveArgs(self):
         config_file = tempfile.NamedTemporaryFile(mode="w", delete=True)
 
@@ -319,8 +369,7 @@ class TestBasicUseCases(TestCase):
 
         self.assertRegex(self.format_help(),
             r'usage: .* \[-h\] --genome GENOME \[-v\]\s+ \(-f1 TYPE1_CFG_FILE \|'
-            ' \\s*-f2 TYPE2_CFG_FILE\\)\\s+\\(-f FRMT \\| -b\\)\n\n' +
-            7*r'.+\s+'+  # repeated 7 times because .+ matches atmost 1 line
+            ' \\s*-f2 TYPE2_CFG_FILE\\)\\s+\\(-f FRMT \\| -b\\)\n\n'
             'optional arguments:\n'
             '  -h, --help            show this help message and exit\n'
             '  -f1 TYPE1_CFG_FILE, --type1-cfg-file TYPE1_CFG_FILE\n'
@@ -329,7 +378,8 @@ class TestBasicUseCases(TestCase):
             '  -b, --bam\\s+\\[env var: BAM_FORMAT\\]\n\n'
             'group1:\n'
             '  --genome GENOME       Path to genome file\n'
-            '  -v\n')
+            '  -v\n\n' +
+            5*r'(.+\s*)')
         config_file.close()
 
     def testSubParsers(self):
@@ -386,6 +436,7 @@ class TestBasicUseCases(TestCase):
         self.add_arg('--c')
         self.add_arg('--b', action="store_true")
         self.add_arg('--a', action="append", type=int)
+        self.add_arg('--m', action="append", nargs=3, metavar=("<a1>", "<a2>", "<a3>"),)
 
         ns = self.parse(args="-x 1", env_vars={}, config_file_contents="""
 
@@ -421,6 +472,8 @@ class TestBasicUseCases(TestCase):
         a = 33
         ---
         z z 1
+        ---
+        m = [[1, 2, 3], [4, 5, 6]]
         """)
 
         self.assertEqual(ns.x, 1)
@@ -436,12 +489,13 @@ class TestBasicUseCases(TestCase):
             '  b: \\s+ True\n'
             '  a: \\s+ 33\n'
             '  z: \\s+ z 1\n')
-
+        self.assertEqual(ns.m, [['1', '2', '3'], ['4', '5', '6']])
 
         # -x is not a long arg so can't be set via config file
         self.assertParseArgsRaises("argument -x is required"
                                    if sys.version_info.major < 3 else
                                    "the following arguments are required: -x, --y",
+                                   args="",
                                    config_file_contents="-x 3")
         self.assertParseArgsRaises("invalid float value: 'abc'",
                                    args="-x 5",
@@ -467,6 +521,46 @@ class TestBasicUseCases(TestCase):
             env_vars={"bla": "2"})
         self.assertEqual(set(args), {"--bla=3", "-x", "1"})
 
+
+    def testQuotedArgumentValues(self):
+        self.initParser()
+        self.add_arg("-a")
+        self.add_arg("--b")
+        self.add_arg("-c")
+        self.add_arg("--d")
+        self.add_arg("-e")
+        self.add_arg("-q")
+        self.add_arg("--quotes")
+
+        # sys.argv equivalent of -a="1"  --b "1" -c= --d "" -e=: -q "\"'" --quotes "\"'"
+        ns = self.parse(args=['-a=1', '--b', '1', '-c=', '--d', '', '-e=:',
+                '-q', '"\'', '--quotes', '"\''],
+                env_vars={}, config_file_contents="")
+
+        self.assertEqual(ns.a, "1")
+        self.assertEqual(ns.b, "1")
+        self.assertEqual(ns.c, "")
+        self.assertEqual(ns.d, "")
+        self.assertEqual(ns.e, ":")
+        self.assertEqual(ns.q, '"\'')
+        self.assertEqual(ns.quotes, '"\'')
+
+    def testQuotedConfigFileValues(self):
+        self.initParser()
+        self.add_arg("--a")
+        self.add_arg("--b")
+        self.add_arg("--c")
+
+        ns = self.parse(args="", env_vars={}, config_file_contents="""
+        a="1"
+        b=:
+        c=
+        """)
+
+        self.assertEqual(ns.a, "1")
+        self.assertEqual(ns.b, ":")
+        self.assertEqual(ns.c, "")
+
     def testBooleanValuesCanBeExpressedAsNumbers(self):
         self.initParser()
         store_true_env_var_name = "STORE_TRUE"
@@ -519,6 +613,7 @@ class TestBasicUseCases(TestCase):
         self.add_arg("-v", "--verbose", env_var="VERBOSE", action="store_true")
         self.assertParseArgsRaises("Unexpected value for VERBOSE: 'bla'. "
                                    "Expecting 'true', 'false', 'yes', 'no', '1' or '0'",
+            args="",
             env_vars={"VERBOSE" : "bla"})
         ns = self.parse("",
                         config_file_contents="verbose=true",
@@ -550,9 +645,10 @@ class TestBasicUseCases(TestCase):
         self.add_arg("--arg3", action='append')
         self.add_arg("--arg4", action='append', env_var="ARG4")
         self.add_arg("--arg5", action='append')
+        self.add_arg("--arg6")
 
         ns = self.parse(
-            "--arg0=-foo --arg3=-foo --arg3=-bar",
+            "--arg0=-foo --arg3=-foo --arg3=-bar --arg6=-test-more-dashes",
             config_file_contents="arg2: -foo\narg5: [-foo, -bar]",
             env_vars={"ARG1": "-foo", "ARG4": "[-foo, -bar]"}
         )
@@ -562,6 +658,7 @@ class TestBasicUseCases(TestCase):
         self.assertEqual(ns.arg3, ["-foo", "-bar"])
         self.assertEqual(ns.arg4, ["-foo", "-bar"])
         self.assertEqual(ns.arg5, ["-foo", "-bar"])
+        self.assertEqual(ns.arg6, "-test-more-dashes")
 
     def testPriorityKnown(self):
         self.initParser()
@@ -657,6 +754,28 @@ class TestBasicUseCases(TestCase):
         self.assertEqual(ns.arg, ['Shell', 'someword', 'anotherword'])
         self.assertEqual(ns.a, "positional_value")
 
+    def testCounterCommandLine(self):
+        self.initParser()
+        self.add_arg("--verbose", "-v", action="count", default=0)
+
+        ns = self.parse(args="-v -v -v", env_vars={})
+        self.assertEqual(ns.verbose, 3)
+
+        ns = self.parse(args="-vvv", env_vars={})
+        self.assertEqual(ns.verbose, 3)
+
+    def testCounterConfigFile(self):
+        self.initParser()
+        self.add_arg("--verbose", "-v", action="count", default=0)
+
+        ns = self.parse(args="", env_vars={}, config_file_contents="""
+        verbose""")
+        self.assertEqual(ns.verbose, 1)
+
+        ns = self.parse(args="", env_vars={}, config_file_contents="""
+        verbose=3""")
+        self.assertEqual(ns.verbose, 3)
+
 class TestMisc(TestCase):
     # TODO test different action types with config file, env var
 
@@ -724,7 +843,8 @@ class TestMisc(TestCase):
         self.add_arg('--genome', help='Path to genome file', required=True)
         self.assertParseArgsRaises("argument -c/--config is required"
                                    if sys.version_info.major < 3 else
-                                   "arguments are required: -c/--config",)
+                                   "arguments are required: -c/--config",
+                                   args="")
 
         temp_cfg2 = tempfile.NamedTemporaryFile(mode="w", delete=True)
         ns = self.parse("-c " + temp_cfg2.name)
@@ -737,15 +857,17 @@ class TestMisc(TestCase):
         self.assertEqual(ns.genome, "hg20")
 
         self.assertRegex(self.format_help(),
-            'usage: .* \\[-h\\] -c CONFIG_FILE --genome GENOME\n\n'+
-            7*r'.+\s+'+  # repeated 7 times because .+ matches atmost 1 line
+            'usage: .* \\[-h\\] -c CONFIG_FILE --genome GENOME\n\n'
             'optional arguments:\n'
             '  -h, --help\\s+ show this help message and exit\n'
             '  -c CONFIG_FILE, --config CONFIG_FILE\\s+ my config file\n'
-            '  --genome GENOME\\s+ Path to genome file\n')
+            '  --genome GENOME\\s+ Path to genome file\n\n' +
+            5*r'(.+\s*)')
 
         # just run print_values() to make sure it completes and returns None
-        self.assertIsNone(self.parser.print_values(file=sys.stderr))
+        output = StringIO()
+        self.assertIsNone(self.parser.print_values(file=output))
+        self.assertIn("Command Line Args:", output.getvalue())
 
         # test ignore_unknown_config_file_keys=False
         self.initParser(ignore_unknown_config_file_keys=False)
@@ -795,14 +917,7 @@ class TestMisc(TestCase):
 
         self.assertRegex(self.format_help(),
             r'usage: .* \[-h\] -c CONFIG_FILE\s+'
-            r'\[-w CONFIG_OUTPUT_PATH\]\s* --arg1 ARG1\s*\[--flag\]\s*'
-            'Args that start with \'--\' \\(eg. --arg1\\) can also be set in a '
-            r'config file\s*\(~/.myconfig or specified via -c\).\s*'
-            r'Config file syntax allows: key=value,\s*flag=true, stuff=\[a,b,c\] '
-            r'\(for details, see syntax at https://goo.gl/R74nmi\).\s*'
-            r'If an arg is specified in more than\s*one place, then '
-            r'commandline values\s*override config file values which override\s*'
-            r'defaults.\s*'
+            r'\[-w CONFIG_OUTPUT_PATH\]\s* --arg1\s+ARG1\s*\[--flag\]\s*'
             r'optional arguments:\s*'
             '-h, --help \\s* show this help message and exit\n\\s*'
             r'-c CONFIG_FILE, --config CONFIG_FILE\s+my config file\s*'
@@ -810,7 +925,14 @@ class TestMisc(TestCase):
             r'the current command line args and writes them\s*'
             r'out to a config file at the given path, then exits\s*'
             r'--arg1 ARG1\s*Arg1 help text\s*'
-            r'--flag \s*Flag help text'
+            r'--flag \s*Flag help text\s*'
+            'Args that start with \'--\' \\(eg. --arg1\\) can also be set in a '
+            r'config file\s*\(~/.myconfig or specified via -c\).\s*'
+            r'Config file syntax allows: key=value,\s*flag=true, stuff=\[a,b,c\] '
+            r'\(for details, see syntax at https://goo.gl/R74nmi\).\s*'
+            r'If an arg is specified in more than\s*one place, then '
+            r'commandline values\s*override config file values which override\s*'
+            r'defaults.'
         )
 
     def test_FormatHelpProg(self):
@@ -872,6 +994,7 @@ class TestMisc(TestCase):
             expected_config_file_contents.strip())
         self.assertRaisesRegex(ValueError, "Couldn't open / for writing:",
             self.parse, args = command_line_args + " -w /")
+        cfg_f.close()
 
     def testConstructor_WriteOutConfigFileArgs2(self):
         # Test constructor args:
@@ -916,6 +1039,7 @@ class TestMisc(TestCase):
                          expected_config_file_contents.strip())
         self.assertRaisesRegex(ValueError, "Couldn't open / for writing:",
                                 self.parse, args = command_line_args + " -w /")
+        cfg_f.close()
 
     def testConstructor_WriteOutConfigFileArgsLong(self):
         """Test config writing with long version of arg
@@ -959,6 +1083,7 @@ class TestMisc(TestCase):
             expected_config_file_contents.strip())
         self.assertRaisesRegex(ValueError, "Couldn't open / for writing:",
             self.parse, args = command_line_args + " --write-config /")
+        cfg_f.close()
 
     def testMethodAliases(self):
         p = self.parser
@@ -986,6 +1111,20 @@ class TestMisc(TestCase):
         options = p.parse(args=[])
         self.assertDictEqual(vars(options), {})
 
+    def testConfigOpenFuncError(self):
+        # test OSError
+        def error_func(path):
+            raise OSError(9, "some error")
+        self.initParser(config_file_open_func=error_func)
+        self.parser.add_argument('-g', is_config_file=True)
+        self.assertParseArgsRaises("Unable to open config file: file.txt. Error: some error", args="-g file.txt")
+
+        # test other error
+        def error_func(path):
+            raise Exception('custom error')
+        self.initParser(config_file_open_func=error_func)
+        self.parser.add_argument('-g', is_config_file=True)
+        self.assertParseArgsRaises("Unable to open config file: file.txt. Error: custom error", args="-g file.txt")
 
 class TestConfigFileParsers(TestCase):
     """Test ConfigFileParser subclasses in isolation"""
@@ -1036,11 +1175,254 @@ class TestConfigFileParsers(TestCase):
             '_b': 'c',
             '_list_arg1': ['a', 'b', 'c'],
             '_str_arg': 'true',
-            '_list_arg2': ['1', '2', '3'],
+            '_list_arg2': [1, 2, 3],
         })
 
         self.assertListEqual(parsed_obj['_list_arg1'], ['a', 'b', 'c'])
-        self.assertListEqual(parsed_obj['_list_arg2'], ['1', '2', '3'])
+        self.assertListEqual(parsed_obj['_list_arg2'], [1, 2, 3])
+
+    def testDefaultConfigFileParser_BasicValues(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key = value # comment # comment',   'expected': ('key', 'value', 'comment # comment')},
+            {'line': 'key=value#comment ',                'expected': ('key', 'value#comment', None)},
+            {'line': 'key=value',                         'expected': ('key', 'value', None)},
+            {'line': 'key =value',                        'expected': ('key', 'value', None)},
+            {'line': 'key= value',                        'expected': ('key', 'value', None)},
+            {'line': 'key = value',                       'expected': ('key', 'value', None)},
+            {'line': 'key  =  value',                     'expected': ('key', 'value', None)},
+            {'line': ' key  =  value ',                   'expected': ('key', 'value', None)},
+            {'line': 'key:value',                         'expected': ('key', 'value', None)},
+            {'line': 'key :value',                        'expected': ('key', 'value', None)},
+            {'line': 'key: value',                        'expected': ('key', 'value', None)},
+            {'line': 'key : value',                       'expected': ('key', 'value', None)},
+            {'line': 'key  :  value',                     'expected': ('key', 'value', None)},
+            {'line': ' key  :  value ',                   'expected': ('key', 'value', None)},
+            {'line': 'key value',                         'expected': ('key', 'value', None)},
+            {'line': 'key  value',                        'expected': ('key', 'value', None)},
+            {'line': ' key    value ',                    'expected': ('key', 'value', None)},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
+
+    def testDefaultConfigFileParser_QuotedValues(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key="value"',                       'expected': ('key', 'value', None)},
+            {'line': 'key  =  "value"',                   'expected': ('key', 'value', None)},
+            {'line': ' key  =  "value" ',                 'expected': ('key', 'value', None)},
+            {'line': 'key=" value "',                     'expected': ('key', ' value ', None)},
+            {'line': 'key  =  " value "',                 'expected': ('key', ' value ', None)},
+            {'line': ' key  =  " value " ',               'expected': ('key', ' value ', None)},
+            {'line': "key='value'",                       'expected': ('key', 'value', None)},
+            {'line': "key  =  'value'",                   'expected': ('key', 'value', None)},
+            {'line': " key  =  'value' ",                 'expected': ('key', 'value', None)},
+            {'line': "key=' value '",                     'expected': ('key', ' value ', None)},
+            {'line': "key  =  ' value '",                 'expected': ('key', ' value ', None)},
+            {'line': " key  =  ' value ' ",               'expected': ('key', ' value ', None)},
+            {'line': 'key="',                             'expected': ('key', '"', None)},
+            {'line': 'key  =  "',                         'expected': ('key', '"', None)},
+            {'line': ' key  =  " ',                       'expected': ('key', '"', None)},
+            {'line': 'key = \'"value"\'',                 'expected': ('key', '"value"', None)},
+            {'line': 'key = "\'value\'"',                 'expected': ('key', "'value'", None)},
+            {'line': 'key = ""value""',                   'expected': ('key', '"value"', None)},
+            {'line': 'key = \'\'value\'\'',               'expected': ('key', "'value'", None)},
+            {'line': 'key="value',                        'expected': ('key', '"value', None)},
+            {'line': 'key  =  "value',                    'expected': ('key', '"value', None)},
+            {'line': ' key  =  "value ',                  'expected': ('key', '"value', None)},
+            {'line': 'key=value"',                        'expected': ('key', 'value"', None)},
+            {'line': 'key  =  value"',                    'expected': ('key', 'value"', None)},
+            {'line': ' key  =  value " ',                 'expected': ('key', 'value "', None)},
+            {'line': "key='value",                        'expected': ('key', "'value", None)},
+            {'line': "key  =  'value",                    'expected': ('key', "'value", None)},
+            {'line': " key  =  'value ",                  'expected': ('key', "'value", None)},
+            {'line': "key=value'",                        'expected': ('key', "value'", None)},
+            {'line': "key  =  value'",                    'expected': ('key', "value'", None)},
+            {'line': " key  =  value ' ",                 'expected': ('key', "value '", None)},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
+
+    def testDefaultConfigFileParser_BlankValues(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key=',                              'expected': ('key', '', None)},
+            {'line': 'key =',                             'expected': ('key', '', None)},
+            {'line': 'key= ',                             'expected': ('key', '', None)},
+            {'line': 'key = ',                            'expected': ('key', '', None)},
+            {'line': 'key  =  ',                          'expected': ('key', '', None)},
+            {'line': ' key  =   ',                        'expected': ('key', '', None)},
+            {'line': 'key:',                              'expected': ('key', '', None)},
+            {'line': 'key :',                             'expected': ('key', '', None)},
+            {'line': 'key: ',                             'expected': ('key', '', None)},
+            {'line': 'key : ',                            'expected': ('key', '', None)},
+            {'line': 'key  :  ',                          'expected': ('key', '', None)},
+            {'line': ' key  :   ',                        'expected': ('key', '', None)},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
+
+    def testDefaultConfigFileParser_UnspecifiedValues(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key ',                              'expected': ('key', 'true', None)},
+            {'line': 'key',                               'expected': ('key', 'true', None)},
+            {'line': 'key  ',                             'expected': ('key', 'true', None)},
+            {'line': ' key     ',                         'expected': ('key', 'true', None)},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
+
+    def testDefaultConfigFileParser_ColonEqualSignValue(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key=:',                             'expected': ('key', ':', None)},
+            {'line': 'key =:',                            'expected': ('key', ':', None)},
+            {'line': 'key= :',                            'expected': ('key', ':', None)},
+            {'line': 'key = :',                           'expected': ('key', ':', None)},
+            {'line': 'key  =  :',                         'expected': ('key', ':', None)},
+            {'line': ' key  =  : ',                       'expected': ('key', ':', None)},
+            {'line': 'key:=',                             'expected': ('key', '=', None)},
+            {'line': 'key :=',                            'expected': ('key', '=', None)},
+            {'line': 'key: =',                            'expected': ('key', '=', None)},
+            {'line': 'key : =',                           'expected': ('key', '=', None)},
+            {'line': 'key  :  =',                         'expected': ('key', '=', None)},
+            {'line': ' key  :  = ',                       'expected': ('key', '=', None)},
+            {'line': 'key==',                             'expected': ('key', '=', None)},
+            {'line': 'key ==',                            'expected': ('key', '=', None)},
+            {'line': 'key= =',                            'expected': ('key', '=', None)},
+            {'line': 'key = =',                           'expected': ('key', '=', None)},
+            {'line': 'key  =  =',                         'expected': ('key', '=', None)},
+            {'line': ' key  =  = ',                       'expected': ('key', '=', None)},
+            {'line': 'key::',                             'expected': ('key', ':', None)},
+            {'line': 'key ::',                            'expected': ('key', ':', None)},
+            {'line': 'key: :',                            'expected': ('key', ':', None)},
+            {'line': 'key : :',                           'expected': ('key', ':', None)},
+            {'line': 'key  :  :',                         'expected': ('key', ':', None)},
+            {'line': ' key  :  : ',                       'expected': ('key', ':', None)},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
+
+    def testDefaultConfigFileParser_ValuesWithComments(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key=value#comment ',                'expected': ('key', 'value#comment', None)},
+            {'line': 'key=value #comment',                'expected': ('key', 'value', 'comment')},
+            {'line': ' key  =  value  #  comment',        'expected': ('key', 'value', 'comment')},
+            {'line': 'key:value#comment',                 'expected': ('key', 'value#comment', None)},
+            {'line': 'key:value #comment',                'expected': ('key', 'value', 'comment')},
+            {'line': ' key  :  value  #  comment',        'expected': ('key', 'value', 'comment')},
+            {'line': 'key=value;comment ',                'expected': ('key', 'value;comment', None)},
+            {'line': 'key=value ;comment',                'expected': ('key', 'value', 'comment')},
+            {'line': ' key  =  value  ;  comment',        'expected': ('key', 'value', 'comment')},
+            {'line': 'key:value;comment',                 'expected': ('key', 'value;comment', None)},
+            {'line': 'key:value ;comment',                'expected': ('key', 'value', 'comment')},
+            {'line': ' key  :  value  ;  comment',        'expected': ('key', 'value', 'comment')},
+            {'line': 'key = value # comment # comment',   'expected': ('key', 'value', 'comment # comment')},
+            {'line': 'key = "value # comment" # comment', 'expected': ('key', 'value # comment', 'comment')},
+            {'line': 'key = "#" ; comment',               'expected': ('key', '#', 'comment')},
+            {'line': 'key = ";" # comment',               'expected': ('key', ';', 'comment')},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
+
+    def testDefaultConfigFileParser_NegativeValues(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key = -10',                       'expected': ('key', '-10', None)},
+            {'line': 'key : -10',                       'expected': ('key', '-10', None)},
+            {'line': 'key -10',                         'expected': ('key', '-10', None)},
+            {'line': 'key = "-10"',                     'expected': ('key', '-10', None)},
+            {'line': "key  =  '-10'",                   'expected': ('key', '-10', None)},
+            {'line': 'key=-10',                         'expected': ('key', '-10', None)},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
+
+    def testDefaultConfigFileParser_KeySyntax(self):
+        p = configargparse.DefaultConfigFileParser()
+
+        # test the all syntax case
+        config_lines = [
+            {'line': 'key_underscore = value',            'expected': ('key_underscore', 'value', None)},
+            {'line': 'key_underscore=',                   'expected': ('key_underscore', '', None)},
+            {'line': 'key_underscore',                    'expected': ('key_underscore', 'true', None)},
+            {'line': '_key_underscore = value',           'expected': ('_key_underscore', 'value', None)},
+            {'line': '_key_underscore=',                  'expected': ('_key_underscore', '', None)},
+            {'line': '_key_underscore',                   'expected': ('_key_underscore', 'true', None)},
+            {'line': 'key_underscore_ = value',           'expected': ('key_underscore_', 'value', None)},
+            {'line': 'key_underscore_=',                  'expected': ('key_underscore_', '', None)},
+            {'line': 'key_underscore_',                   'expected': ('key_underscore_', 'true', None)},
+            {'line': 'key-dash = value',                  'expected': ('key-dash', 'value', None)},
+            {'line': 'key-dash=',                         'expected': ('key-dash', '', None)},
+            {'line': 'key-dash',                          'expected': ('key-dash', 'true', None)},
+            {'line': 'key@word = value',                  'expected': ('key@word', 'value', None)},
+            {'line': 'key@word=',                         'expected': ('key@word', '', None)},
+            {'line': 'key@word',                          'expected': ('key@word', 'true', None)},
+            {'line': 'key$word = value',                  'expected': ('key$word', 'value', None)},
+            {'line': 'key$word=',                         'expected': ('key$word', '', None)},
+            {'line': 'key$word',                          'expected': ('key$word', 'true', None)},
+            {'line': 'key.word = value',                  'expected': ('key.word', 'value', None)},
+            {'line': 'key.word=',                         'expected': ('key.word', '', None)},
+            {'line': 'key.word',                          'expected': ('key.word', 'true', None)},
+        ]
+
+        for test in config_lines:
+            parsed_obj = p.parse(StringIO(test['line']))
+            parsed_obj = dict(parsed_obj)
+            expected = {test['expected'][0]: test['expected'][1]}
+            self.assertDictEqual(parsed_obj, expected,
+                    msg="Line %r" % (test['line']))
 
     def testYAMLConfigFileParser_Basic(self):
         try:
@@ -1111,7 +1493,14 @@ except ImportError:
 else:
     test_argparse_source_code = inspect.getsource(test.test_argparse)
     test_argparse_source_code = test_argparse_source_code.replace(
-        'argparse.ArgumentParser', 'configargparse.ArgumentParser')
+        'argparse.ArgumentParser', 'configargparse.ArgumentParser').replace(
+        'TestHelpFormattingMetaclass', '_TestHelpFormattingMetaclass').replace(
+        'test_main', '_test_main')
+
+    # pytest tries to collect tests from TestHelpFormattingMetaclass, and
+    # test_main, and raises a warning when it finds it's not a test class
+    # nor test function. Renaming TestHelpFormattingMetaclass and test_main
+    # prevents pytest from trying.
 
     # run or debug a subset of the argparse tests
     #test_argparse_source_code = test_argparse_source_code.replace(
